// ProjectManager.cpp

#include "ProjectManager.h"
#include "../../core/utils/EngineConfig.h"
#include "../../core/utils/Logging.h"
#include "../../core/utils/AssetManager.h"
#include <yaml-cpp/yaml.h>
#include <fstream>
#include <chrono>

namespace fs = std::filesystem;

std::string ProjectManager::s_currentProjectPath;
std::string ProjectManager::s_currentProjectName;

bool ProjectManager::Init()
{
    s_currentProjectPath.clear();
    s_currentProjectName.clear();
    Logger::LogOk("Project Core initialized");
    return true;
}

bool ProjectManager::SaveResource()
{
    return false;
}

bool ProjectManager::CreateProject(const std::string &projectPath,
                                   const std::string &projectName)
{
    fs::path baseDir = fs::path(projectPath) / projectName;
    fs::path projFile = baseDir / (projectName + ".cproj");
    std::error_code ec;

    if (fs::exists(projFile))
    {
        YAML::Node config;
        try
        {
            config = YAML::LoadFile(projFile.string());
        }
        catch (const YAML::Exception &e)
        {
            Logger::LogError("Failed to parse existing project file '%s': %s",
                             projFile.string().c_str(), e.what());
            return false;
        }

        std::string savedName = config["Name"].as<std::string>(projectName);
        std::string createdDate = config["CreatedDate"].as<std::string>("unknown");

        Logger::LogInfo("Loaded project '%s' at '%s' (created %s)",
                        savedName.c_str(),
                        projectPath.c_str(),
                        createdDate.c_str());

        s_currentProjectPath = projectPath;
        s_currentProjectName = savedName;

        fs::create_directories(baseDir / "asset" / "Prefabs", ec);
        if (ec)
            Logger::LogError("Could not ensure Prefabs dir: %s",
                             ec.message().c_str());

        return true;
    }

    if (!fs::create_directories(baseDir, ec) && ec)
    {
        Logger::LogError("Failed to create project directory '%s': %s",
                         projectPath.c_str(), ec.message().c_str());
        return false;
    }

    YAML::Emitter out;
    out << YAML::BeginMap;
    out << YAML::Key << "Name" << YAML::Value << projectName;
    out << YAML::Key << "EngineVersion" << YAML::Value << g_engineConfig.version;

    auto now = std::chrono::system_clock::now();
    auto t = std::chrono::system_clock::to_time_t(now);
    std::tm tm{};
#if defined(_WIN32)
    gmtime_s(&tm, &t);
#else
    gmtime_r(&t, &tm);
#endif
    char buf[32];
    std::strftime(buf, sizeof(buf), "%Y-%m-%dT%H:%M:%SZ", &tm);
    out << YAML::Key << "CreatedDate" << YAML::Value << buf;
    out << YAML::EndMap;

    std::ofstream fout(projFile);
    if (!fout.is_open())
    {
        Logger::LogError("Could not open project file for writing: %s",
                         projFile.string().c_str());
        return false;
    }
    fout << out.c_str();
    fout.close();

    Logger::LogOk("Created new project '%s' at '%s'",
                  projectName.c_str(), projectPath.c_str());

    fs::create_directories(baseDir / "asset" / "Prefabs", ec);

    if (ec)
        Logger::LogError("Could not create Prefabs dir: %s",
                         ec.message().c_str());

    s_currentProjectPath = projectPath;
    s_currentProjectName = projectName;
    return true;
}

const std::string &ProjectManager::GetCurrentProjectPath()
{
    return s_currentProjectPath;
}

const std::string &ProjectManager::GetCurrentProjectName()
{
    return s_currentProjectName;
}

const std::string &ProjectManager::GetCurrentAssetsPath()
{
    static std::string assetsPath;
    // <projectPath>/<projectName>/Assets
    fs::path p = fs::path(s_currentProjectPath) / s_currentProjectName / "Assets";
    assetsPath = p.string();
    return assetsPath;
}
